@@ -4,11 +4,11 @@
 
 /*WIN32DLL*/
 /* $B$3$A$i$N%P!<%8%g%s$b99?7$7$F$/$@$5$$!#(B */
-#define NKF_VERSIONW L"2.0.8"
+#define NKF_VERSIONW L"2.0.9"
 /* NKF_VERSION $B$N%o%$%IJ8;z(B */
-#define DLL_VERSION   "2.0.8.0 1"
+#define DLL_VERSION   "2.0.9.0 0"
 /* DLL$B$,JV$9(B */
-#define DLL_VERSIONW L"2.0.8.0 1"
+#define DLL_VERSIONW L"2.0.9.0 0"
 /* DLL$B$,JV$9(B DLL_VERSION $B$N%o%$%IJ8;z(B */
 
 /* nkf32.dll main */
@@ -53,7 +53,7 @@ wchar_t *guessbuffW = NULL;
 UINT guessCodePage = CP_OEMCP;
 DWORD guessdwFlags = MB_PRECOMPOSED;
 
-wchar_t *tounicode(char *p)
+wchar_t *tounicode(const char *p)
 {
 static wchar_t buff[GUESS];
     int sts;
@@ -182,14 +182,7 @@ void
 print_guessed_code (filename)
     char *filename;
 {
-    char *codename = "BINARY";
-    if (!is_inputcode_mixed) {
-        if (strcmp(input_codename, "") == 0) {
-            codename = "ASCII";
-        } else {
-            codename = input_codename;
-        }
-    }
+    const char *codename = get_guessed_code();
     if (filename != NULL) {
         guessbuffA = realloc(guessbuffA,(strlen(filename) + GUESS + 1) * sizeof (char) );
         sprintf(guessbuffA,"%s:%s", filename,codename);
@@ -204,20 +197,16 @@ void
 print_guessed_codeW (filename)
     wchar_t *filename;
 {
-    char *codename = "BINARY";
-    if (!is_inputcode_mixed) {
-        if (strcmp(input_codename, "") == 0) {
-            codename = "ASCII";
-        } else {
-            codename = input_codename;
-        }
-    }
+    const char *codename = get_guessed_code();
+    size_t size;
     if (filename != NULL) {
-        guessbuffW = realloc(guessbuffW,(wcslen(filename) + GUESS + 1) * sizeof (wchar_t) );
-        swprintf(guessbuffW,L"%s:%s",filename,tounicode(codename));
+	size = (wcslen(filename) + GUESS + 1) * sizeof (wchar_t);
+	guessbuffW = realloc(guessbuffW, size);
+	_snwprintf(guessbuffW, size, L"%s:%s", filename, tounicode(codename));
     } else {
-        guessbuffW = realloc(guessbuffW,(GUESS + 1) * sizeof (wchar_t));
-        swprintf(guessbuffW,L"%s",tounicode(codename));
+	size = (GUESS + 1) * sizeof (wchar_t);
+	guessbuffW = realloc(guessbuffW, size);
+	_snwprintf(guessbuffW, size, L"%s", tounicode(codename));
     }
 }
 #endif /*UNICODESUPPORT*/
@@ -236,8 +225,8 @@ print_guessed_codeW (filename)
  **
  **/
 
-void 
-reinitdll()
+void
+reinitdll(void)
 {
     cin = NULL;
     nin = -1;
@@ -331,27 +320,38 @@ BOOL WINAPI GetNkfVersionSafeW(LPWSTR verStr,DWORD nBufferLength /*in TCHARs*/,L
 #endif /*UNICODESUPPORT*/
 }
 
-int CALLBACK SetNkfOption(LPCSTR optStr)
+static LPSTR optStr0 = NULL;
+
+int CALLBACK SetNkfOption(LPSTR optStr)
 {
     LPSTR p;
+    int len;
 
     if ( *optStr == '-' ) {
-        reinit();
-        options(optStr);
+        len = strlen(optStr) + 1;
+        p = realloc(optStr0,len);
+        strcpy(p,optStr);
     } else {
-        p = malloc(strlen(optStr) + 2);
-        if ( p == NULL ) return -1;
+        len = strlen(optStr) + 2;
+        p = realloc(optStr0,len);
         *p = '-';
         strcpy(p + 1,optStr);
-        reinit();
-        options(p);
-        free(p);
     }
+    optStr0 = p;
     return 0;
 }
 
+void options0(void)
+{
+    reinit();
+    if ( optStr0 != NULL ) {
+        options(optStr0);
+    }
+}
+
 void CALLBACK NkfConvert(LPSTR outStr, LPCSTR inStr)
 {
+    options0();
     std_putc_mode = 2;
     cout = outStr;
     noutmax = -1;
@@ -366,6 +366,7 @@ void CALLBACK NkfConvert(LPSTR outStr, LPCSTR inStr)
 
 BOOL WINAPI NkfConvertSafe(LPSTR outStr,DWORD nOutBufferLength /*in Bytes*/,LPDWORD lpBytesReturned /*in Bytes*/, LPCSTR inStr,DWORD nInBufferLength /*in Bytes*/){
     if ( inStr == NULL ) return FALSE;
+    options0();
     std_putc_mode = 6;
     cout = outStr;
     noutmax = nOutBufferLength;
@@ -389,7 +390,6 @@ void CALLBACK ToHankaku(LPSTR inStr)
     p = malloc(len);
     if ( p == NULL ) return;
     memcpy(p,inStr,len);
-    reinit();
     options("-ZSs");
     NkfConvert(inStr,p);
     free(p);
@@ -397,34 +397,29 @@ void CALLBACK ToHankaku(LPSTR inStr)
 
 BOOL WINAPI ToHankakuSafe(LPSTR outStr,DWORD nOutBufferLength /*in Bytes*/,LPDWORD lpBytesReturned /*in Bytes*/,LPCSTR inStr,DWORD nInBufferLength /*in Bytes*/)
 {
-    reinit();
     options("-ZSs");
     return NkfConvertSafe(outStr,nOutBufferLength,lpBytesReturned,inStr,nInBufferLength);
 }
 
 void CALLBACK ToZenkakuKana(LPSTR outStr, LPCSTR inStr)
 {
-    reinit();
     options("-Ss");
     NkfConvert(outStr, inStr);
 }
 
 BOOL WINAPI ToZenkakuKanaSafe(LPSTR outStr,DWORD nOutBufferLength /*in Bytes*/,LPDWORD lpBytesReturned /*in Bytes*/,LPCSTR inStr,DWORD nInBufferLength /*in Bytes*/)
 {
-    reinit();
     options("-Ss");
     return NkfConvertSafe(outStr,nOutBufferLength,lpBytesReturned,inStr,nInBufferLength);
 }
 
 void CALLBACK EncodeSubject(LPSTR outStr ,LPCSTR inStr){
-    reinit();
     options("-jM");
     NkfConvert(outStr, inStr);
 }
 
 BOOL WINAPI EncodeSubjectSafe(LPSTR outStr,DWORD nOutBufferLength /*in Bytes*/,LPDWORD lpBytesReturned /*in Bytes*/,LPCSTR inStr,DWORD nInBufferLength /*in Bytes*/)
 {
-    reinit();
     options("-jM");
     return NkfConvertSafe(outStr,nOutBufferLength,lpBytesReturned,inStr,nInBufferLength);
 }
@@ -756,7 +751,7 @@ BOOL WINAPI GetNkfSupportFunctions(void *outStr,DWORD nBufferLength /*in Bytes*/
 {
     *lpBytesReturned = sizeof NkfSupportFunctions;
     if ( outStr == NULL || nBufferLength == 0 ) return FALSE;
-    NkfSupportFunctions.copyrightA = CopyRight;
+    NkfSupportFunctions.copyrightA = COPY_RIGHT;
     memcpy(outStr,&NkfSupportFunctions,sizeof NkfSupportFunctions > nBufferLength ? nBufferLength : sizeof NkfSupportFunctions);
     return TRUE;
 }
